#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/16
#Last Modify: 2018/04/16
# deeper learning insights machine learning page 195

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Load the data
df = pd.read_csv('/home/egg/projects/deeper_insights_machine_learning/wdbc.data',
header=None)
# Next, we assign the 30 features to a NumPy array X. Using LabelEncoder, we
# transform the class labels from their original string representation (M and
# B) into integers:
from sklearn.preprocessing import LabelEncoder
X = df.loc[:, 2:].values
y = df.loc[:, 1].values
le = LabelEncoder()
y = le.fit_transform(y)
print le.transform(['M', 'B'])

# Split
from sklearn.cross_validation import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, 
                                                    test_size=0.20,
                                                    random_state=1)
## Create the pipeline
from sklearn.grid_search import GridSearchCV
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.svm import SVC
pipe_svc = Pipeline([('scl', StandardScaler()),
                     ('clf', SVC(random_state=1))])

# Confusion matrix
from sklearn.metrics import confusion_matrix
pipe_svc.fit(X_train, y_train)
y_pred = pipe_svc.predict(X_test)


# Metrics
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score, f1_score
print('Precision: %.3f' % precision_score(
y_true=y_test, y_pred=y_pred))
#Precision: 0.976
print('Recall: %.3f' % recall_score(
y_true=y_test, y_pred=y_pred))
#Recall: 0.952
print('F1: %.3f' % f1_score(
y_true=y_test, y_pred=y_pred))
#F1: 0.964



## Receiver operator characteristic (ROC) / area under the curve (AUC) ########
# By executing the following code example, we will plot an ROC curve of a
# classifier that only uses two features from the Breast Cancer Wisconsin
# dataset to predict whether a tumor is benign or malignant. Although we are
# going to use the same logistic regression pipeline that we defined
# previously, we are making the classification task more challenging for the
# classifier so that the resulting ROC curve becomes visually more interesting.
# For similar reasons, we are also reducing the number of folds in the
# StratifiedKFold validator to three. The code is as follows:
from sklearn.decomposition import PCA
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import roc_curve, auc
from sklearn.cross_validation import StratifiedKFold
from scipy import interp
pipe_lr = Pipeline([('scl', StandardScaler()),
                    ('pca', PCA(n_components=2)),
                    ('clf', LogisticRegression(penalty='l2',
                    random_state=0,
                    C=100.0))])
X_train2 = X_train[:, [4, 14]] #use only 2 features, illustrative ROC
#X_train2 = X_train[:, :] #use all the features, much better ROC
cv = StratifiedKFold(y_train,
                     n_folds=3,
                     random_state=1)
fig = plt.figure(figsize=(7, 5))
mean_tpr = 0.0
mean_fpr = np.linspace(0, 1, 100)
all_tpr = []
for i, (train, test) in enumerate(cv):
  probas = pipe_lr.fit(X_train2[train],
                       y_train[train]).predict_proba(X_train2[test])
  fpr, tpr, thresholds = roc_curve(y_train[test],
                                   probas[:, 1],
                                   pos_label=1)
  mean_tpr += interp(mean_fpr, fpr, tpr)
  mean_tpr[0] = 0.0
  roc_auc = auc(fpr, tpr)
  plt.plot(fpr,
           tpr,
           lw=1,
           label='ROC fold %d (area = %0.2f)'% (i+1, roc_auc))
plt.plot([0, 1],
         [0, 1],
         linestyle='--',
         color=(0.6, 0.6, 0.6),
         label='random guessing')
mean_tpr /= len(cv)
mean_tpr[-1] = 1.0
mean_auc = auc(mean_fpr, mean_tpr)
plt.plot(mean_fpr, mean_tpr, 'k--',
         label='mean ROC (area = %0.2f)' % mean_auc, lw=2)
plt.plot([0, 0, 1],
         [0, 1, 1],
         lw=2,
         linestyle=':',
         color='black',
         label='perfect performance')
plt.xlim([-0.05, 1.05])
plt.ylim([-0.05, 1.05])
plt.xlabel('false positive rate')
plt.ylabel('true positive rate')
plt.title('Receiver Operator Characteristic')
plt.legend(loc="lower right")
plt.show()
# In the preceding code example, we used the already familiar StratifiedKFold
# class from scikit-learn and calculated the ROC performance of the
# LogisticRegression classifier in our pipe_lr pipeline using the roc_curve
# function from the sklearn.metrics module separately for each iteration.
# Furthermore, we interpolated the average ROC curve from the three folds via
# the interp function that we imported from SciPy and calculated the area under
# the curve via the auc function.

# If we are just interested in the ROC AUC score
pipe_lr = pipe_lr.fit(X_train2, y_train)
y_pred2 = pipe_lr.predict(X_test[:, [4, 14]])
from sklearn.metrics import roc_auc_score
from sklearn.metrics import accuracy_score
print
print('ROC AUC: %.3f' % roc_auc_score(
y_true=y_test, y_score=y_pred2))
#ROC AUC: 0.662
print('Accuracy: %.3f' % accuracy_score(y_true=y_test, y_pred=y_pred2))
#Accuracy: 0.711

